{
  Fix999 -- Correct bogus SHIPXY files made by Winplan999/Unpack999

  Tim's Winplan and Unpack programs for Host999 always unpack 999 ships
  to SHIPXYx.DAT, even if the RST was generated by `Host500'.

  This confuses PCC 1.0.11.  Fix999 clears the bogus entries.  When
  applied to a correct file, this will do nothing, so it does not harm
  to run this program after any unpack operation.  This program can
  be invoked with a directory name as parameter (`fix999 game3'), in
  which case it processes all SHIPXY files in that directory; or with
  a single file name (`fix999 game3\shipxy9.dat').

  Winplan users can arrange that this be done automatically:
  + copy this program to your Winplan directory (where you have winplan.exe)
  + in each of the vpwork directories, create a file `unproc2.bat' with
    the command `fix999 vpworkX', where vpworkX is the directory name
    (if that file already exists, add this command at the end).

  11/Jun/2001 by Stefan Reuther <Streu@gmx.de>. Public Domain.
  Turbo Pascal 6.0. No warranties.
}
PROGRAM Fix999;
{$I-}

VAR s     : STRING;
    s1    : STRING[10];
    count : INTEGER;            { Number of files successfully opened }
    i     : INTEGER;

VAR data : ARRAY[1..499*4] OF INTEGER;      { 4k buffer }

{
  Fix SHIPXY file /name/.
}
PROCEDURE FixFile(name : STRING);
VAR i    : INTEGER;
    F    : FILE;
BEGIN
  Assign(F, name);
  Reset(F, 1);
  IF IOResult<>0 THEN Exit;
  Inc(count);

  IF (FileSize(F) >= 7992) THEN BEGIN
   { Not all programs I know write the 10 checksum bytes at the end }
    Blockread(F, data, 8);
    IF (data[1] < 0) OR (data[1] >= $3030) OR
       (data[2] < 0) OR (data[2] >= $3030) OR
       (data[3] < 0) OR (data[3] > 12) THEN
    BEGIN
     { This check should separate out other files, if you accidentially do }
     { `fix999 ship9.dat'. Not bullet-proof, though. }
      Writeln(name, ': seems not to be a SHIPXY file -- not touching it');
    END ELSE BEGIN
      Seek(F, 500*8);
      Blockread(F, data, 8);
      IF (data[1] < 0) OR (data[1] >= $3030) OR
         (data[2] < 0) OR (data[2] >= $302D) THEN BEGIN
       { Bogus shipxy files start with the first four places of the }
       { timestamp at ship #501, i.e. `06-0' from `06-04-2001'. }
       { Interpreted as X/Y coordinates, this is very unlikely. }
       { Checking the owner would sound worthwhile, but unpack is }
       { "smart" enough to zero it out if it is out of range. }
        Writeln(name, ': fixing bogus file');
        FillChar(data, Sizeof(data), 0);
        Seek(F, 500*8);
        Blockwrite(F, data, Sizeof(data));
      END ELSE BEGIN
        Writeln(name, ': seems to be a real Host999 file');
      END;
    END;
  END ELSE BEGIN
    Writeln(name, ': not a Host999 file');
  END;
  Close(F);
END;

BEGIN
  Writeln('Fix999 -- Correct Winplan999/Unpack999 SHIPXY files -- 2001 by Streu');
  Writeln;
  s := ParamStr(1);
  IF s='' THEN BEGIN
    Writeln('Usage: FIX999 <directory>');
    Writeln('  or   FIX999 <filename>');
    Halt;
  END;
  count := 0;
  FixFile(s);
  IF (s[Length(s)]<>'\') THEN s := s + '\';
  FOR i := 1 TO 11 DO BEGIN
    Str(i, s1);
    FixFile(s + 'shipxy' + s1 + '.dat');
  END;

  IF count=0 THEN BEGIN
    Writeln('No files found which match `' + ParamStr(1) + '''.');
  END;
END.
